/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package p4uclient;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;

/**
 * ファイル管理
 * @author tondol
 */
public class FileManager {
    private String filename = null;
    private ArrayList<FileItem> items = null;
    
    /**
     * <送信リスト>を読み出す
     */
    public FileManager(String filename) {
        this.filename = filename;
        items = new ArrayList<FileItem>();
        FileReader     ifstream = null;
        BufferedReader reader   = null;
        try {
            ifstream = new FileReader(filename);
            reader   = new BufferedReader(ifstream);
            while (reader.ready()) {
                String line = reader.readLine();
                addItem(new FileItem(line));
            }
            MainApp.window.putMessage("o リスト読込");
        } catch(Exception e) {
            MainApp.window.putMessage("x リスト読込");
            MainApp.window.putMessage("debug: " + e);
        } finally {
            try {
                if (reader   != null) reader.close();
                if (ifstream != null) ifstream.close();
            } catch(Exception e) {
            }
        }
    }
    
    /**
     * <送信リスト>を永続化
     */
    public void close() throws Exception {
        FileWriter  ofstream = null;
        PrintWriter writer   = null;
        try {
            ofstream = new FileWriter(filename);
            writer   = new PrintWriter(ofstream);
            for (int i=0;i<items.size();i++) {
                FileItem item = items.get(i);
                writer.println(item.toString());
            }
            MainApp.window.putMessage("o リスト書込");
        } catch(Exception e) {
            MainApp.window.putMessage("x リスト書込");
        } finally {
            try {
                if (writer   != null) writer.close();
                if (ofstream != null) ofstream.close();
                clearItems();
            } catch(Exception e) {
            }
        }
    }
    
    /**
     * <ハッシュ>から検索
     */
    public FileItem getItem(String digest) {
        FileItem ret = null;
        for (int i=0;i<items.size();i++){
            if (digest.equals(items.get(i).hash)) {
                ret = items.get(i);
            }
        }
        return ret;
    }
    
    /**
     * <アイテム>を追加
     */
    private void addItem(FileItem item) throws Exception {
        items.add(item);
        MainApp.client.addItem(item);
    }
    
    /**
     * <アイテム>を削除
     */
    private void deleteItem(FileItem item) throws Exception {
        items.remove(item);
        MainApp.client.delItem(item);
    }
    
    /**
     * <アイテム>をクリア
     */
    public void clearItems() throws Exception {
        while (!items.isEmpty()) {
            int index = items.size() - 1;
            deleteItem(items.get(index));
        }
    }
    
    /**
     * <送信リスト>を更新
     */
    public void updateItems() throws Exception {
        // 送信リストを取得
        File directory = new File(MainApp.option.updir);
        File array[]   = directory.listFiles();
        // tmp = items.clone
        ArrayList<FileItem> tmp = new ArrayList<FileItem>();
        for (int i=0;i<items.size();i++) {
            tmp.add(items.get(i));
        }
        // items -= (array - tmp)
        for (int i=0;i<tmp.size();i++) {
            FileItem item = tmp.get(i);
            if (!contains(array, item)) {
                deleteItem(item);
            }
        }
        // items += (tmp - array)
        for (int i=0;i<array.length;i++) {
            File item = array[i];
            if (!contains(tmp, item)) {
                addItem(new FileItem(item));
            }
        }
    }
    
    /**
     * <FileItem>の配列リストに<File>が含まれるか?
     */
    private boolean contains(ArrayList<FileItem> list, File file) {
        for (int i=0;i<list.size();i++) {
            FileItem tmp = list.get(i);
            if (!tmp.name.equals(file.getName())) continue;
            if (!tmp.host.equals(MainApp.client.host)) continue;
            if (tmp.size != file.length()) continue;
            if (tmp.port != MainApp.client.listen) continue;
            return true;
        }
        return false;
    }
    
    /**
     * <File>の配列に<FileItem>が含まれるか?
     */
    private boolean contains(File[] list, FileItem item) {
        for (int i=0;i<list.length;i++) {
            File tmp = list[i];
            if (!tmp.getName().equals(item.name)) continue;
            if (!MainApp.client.host.equals(item.host)) continue;
            if (tmp.length() != item.size) continue;
            if (MainApp.client.listen != item.port) continue;
            return true;
        }
        return false;
    }
}
